package nckd.jimin.jyyy.fi.mservice.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.logging.Log;
import kd.bos.logging.LogFactory;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.QueryServiceHelper;

import kd.bos.util.StringUtils;
import nckd.jimin.jyyy.fi.mservice.SynSapService;
import nckd.jimin.jyyy.fi.mservice.SyncSapFIUtils;

import java.util.*;


/**
 * 同步SAP凭证
 *
 * @author turborao
 * @date 2025/5/12 19:38
 */
public class SynSapFIServiceImpl implements SynSapService {
    private static final Log logger = LogFactory.getLog(SynSapFIServiceImpl.class);


    /**
     * 同步SAP凭证
     *
     * 本方法负责从苍穹系统同步凭证到SAP凭证中
     *
     * @param  vouchsreIds 公司代码，用于过滤特定公司的项目信息
     * @return 返回一个包含同步结果的Map，包括状态码（code）和消息（msg）
     */
    @Override
    public Map<String, String> synVoucherForSap(List<Long> vouchsreIds) {
        Map<String, String> result = new HashMap<>();
        StringBuilder err = new StringBuilder();
        int count = 0;
        for (Long vouchsreId : vouchsreIds) {

            QFilter qf1 = new QFilter("billstatus", QCP.equals, "C");
            QFilter qf2 = new QFilter("id", QCP.equals, vouchsreId);

            String selectFields = SyncSapFIUtils.getVoucherFieldForQuery();
            DynamicObjectCollection voucherDyns = QueryServiceHelper.query("gl_voucher", selectFields, new QFilter[]{qf1, qf2});

            JSONArray ENTRY_ITEM = new JSONArray();  //凭证分录
            JSONObject HEADERITEMINFO = new JSONObject();  //凭证表头
            //现金流量项目编码 不在现金科目行上，需要单独处理下
            String maincfitemNumber = "";
            String voucherInfo = "";

            //voucherDyns 凭证表头与分录平铺
            for (DynamicObject voucherRow : voucherDyns) {
                String billid = voucherRow.getString("id");
                String companyCode = voucherRow.getString("org.number");

                voucherInfo = companyCode + "|" + voucherRow.getString("vouchertype") + "|" + voucherRow.getString("billno");
                Map<String, String> auxpropertiesMap = SyncSapFIUtils.getAssGrp(voucherRow);

                if (StringUtils.isNotEmpty(auxpropertiesMap.get("maincfitemNumber"))) {
                    maincfitemNumber = auxpropertiesMap.get("maincfitemNumber");
                }
                ////处理凭证表头
                HEADERITEMINFO = SyncSapFIUtils.convertCosmicVoucherToSAPForHead(voucherRow);
                JSONObject entry = SyncSapFIUtils.convertCosmicVoucherToSAPForEntry(voucherRow, maincfitemNumber, auxpropertiesMap);
                ENTRY_ITEM.add(entry);

            }

            String jsonData = SyncSapFIUtils.getVoucherData(HEADERITEMINFO, ENTRY_ITEM);
            System.out.println("凭证 info:" + voucherInfo);
            System.out.println("凭证 post:" + jsonData);
            String response = SyncSapFIUtils.postDataToSAP(jsonData);
            //String response = "{\"E_TYPE\":\"S\",\"E_MESSGE\":\"成功\"}";
            System.out.println("凭证 return:" + response);
            if (StringUtils.isEmpty(response)) {
                err.append("同步失败,凭证:" + voucherInfo);
            }
            JSONObject rData = JSONObject.parseObject(response);
            String etype = rData.getString("E_TYPE");

            if (!"S".equals(etype)) {
                err.append("/n 凭证:" + voucherInfo + "，E_TYPE:" + etype + "，错误信息：" + rData.getString("E_MESSGE"));
            } else {
                count++;
            }
        }

        if(StringUtils.isEmpty(err.toString())) {
            result.put("code", "200");
            result.put("msg", "同步SAP凭证，执行成功，本次同步数量"+count);
        }else{
            result.put("code", "300");
            result.put("msg", "同步SAP凭证，本次同步成功数量"+count +"，失败信息："+ err);
        }

        return result;
    }

}
